Package: txdbmaker
Title: Tools for making TxDb objects from genomic annotations
Description: A set of tools for making TxDb objects from genomic annotations
	from various sources (e.g. UCSC, Ensembl, and GFF files). These tools
	allow the user to download the genomic locations of transcripts,
	exons, and CDS, for a given assembly, and to import them in a TxDb
	object. TxDb objects are implemented in the GenomicFeatures package,
	together with flexible methods for extracting the desired features in
	convenient formats.
biocViews: Infrastructure, DataImport, Annotation, GenomeAnnotation,
        GenomeAssembly, Genetics, Sequencing
URL: https://bioconductor.org/packages/txdbmaker
BugReports: https://github.com/Bioconductor/txdbmaker/issues
Version: 1.8.0
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("H.", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"),
	person("M.", "Carlson", role="aut"),
	person("P.", "Aboyoun", role="aut"),
	person("S.", "Falcon", role="aut"),
	person("M.", "Morgan", role="aut"),
	person("R.", "Castelo", role="ctb"),
	person("M.", "Lawrence", role="ctb"),
	person("I-Hsuan", "Lin", role="ctb"),
	person("J.", "MacDonald", role="ctb"),
	person("M.", "Ramos", role="ctb"),
	person("S.", "Saini", role="ctb"),
	person("L.", "Shepherd", role="ctb"))
Depends: BiocGenerics, S4Vectors (>= 0.47.6), Seqinfo, GenomicRanges
        (>= 1.61.1), GenomicFeatures (>= 1.61.4)
Imports: methods, utils, stats, tools, httr, rjson, DBI, RSQLite (>=
        2.0), IRanges, UCSC.utils (>= 1.7.1), GenomeInfoDb,
        AnnotationDbi, Biobase, BiocIO, rtracklayer, biomaRt (>=
        2.59.1)
Suggests: RMariaDB, ensembldb, GenomeInfoDbData, RUnit, BiocStyle,
        knitr
VignetteBuilder: knitr
Collate: utils.R Ensembl-utils.R findCompatibleMarts.R TxDb-schema.R
        TxDb-CREATE-TABLE-helpers.R makeTxDb.R makeTxDbFromUCSC.R
        makeTxDbFromBiomart.R makeTxDbFromEnsembl.R
        makeTxDbFromGRanges.R makeTxDbFromGFF.R makeFeatureDbFromUCSC.R
        makeTxDbPackage.R zzz.R
git_url: https://git.bioconductor.org/packages/txdbmaker
git_branch: RELEASE_3_23
git_last_commit: b7845a9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:18:36 UTC; biocbuild
Author: H. Pagès [aut, cre],
  M. Carlson [aut],
  P. Aboyoun [aut],
  S. Falcon [aut],
  M. Morgan [aut],
  R. Castelo [ctb],
  M. Lawrence [ctb],
  I-Hsuan Lin [ctb],
  J. MacDonald [ctb],
  M. Ramos [ctb],
  S. Saini [ctb],
  L. Shepherd [ctb]
Maintainer: H. Pagès <hpages.on.github@gmail.com>
