[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**txdbmaker** is an R/Bioconductor package that provides a set of tools for making TxDb objects from genomic annotations from various sources (e.g. UCSC, Ensembl, and GFF files).

See https://bioconductor.org/packages/txdbmaker for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

