DB_TYPE_NAME           <- GenomicFeatures:::DB_TYPE_NAME
DB_TYPE_VALUE          <- GenomicFeatures:::DB_TYPE_VALUE
DB_SCHEMA_VERSION      <- GenomicFeatures:::DB_SCHEMA_VERSION
TXDB_CHROMINFO_COLDEFS <- GenomicFeatures:::TXDB_CHROMINFO_COLDEFS
TXDB_FEATURE_COLDEFS   <- GenomicFeatures:::TXDB_FEATURE_COLDEFS
TXDB_SPLICING_COLDEFS  <- GenomicFeatures:::TXDB_SPLICING_COLDEFS
TXDB_GENE_COLDEFS      <- GenomicFeatures:::TXDB_GENE_COLDEFS
TXDB_COLUMNS           <- GenomicFeatures:::TXDB_COLUMNS
TXDB_table_columns     <- GenomicFeatures:::TXDB_table_columns

